{% extends "default_template.html" %}


{% block body %}
    <h1>Pause</h1>

    {% if success %}
    <div class="alert alert-success" role="alert">
        Pause requested; check <a href="{{ url_for('_system') }}">system status</a>.
    </div>
    {% else %}
        <div class="alert alert-secondary" role="alert">
            Pause error: {{error}}
        </div>
    {% endif %}
{% endblock %}

